/*
 * This is part of the doxml library, a parser for XML.  It should not
 *  require any functionality not defined in ANSI C's standard libraries.
 *
 * Copyright (C) 1999 John Stracke, http://www.thibault.org
 *
 * This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Log: tagmatch.c,v $
 * Revision 1.6  1999/07/24 16:55:28  francis
 * Adding support for entity-valued attributes
 *
 * Revision 1.5  1999/07/24 15:23:03  francis
 * Adding support for default attribute values
 *
 * Revision 1.4  1999/07/24 13:13:39  francis
 * Fixing the problem where <D:foo> can be terminated by </foo> instead of
 * by </D:foo>.
 *
 * Revision 1.3  1999/06/01 02:44:41  francis
 * Changing doxml_tags_match() so that namespaces come before tags, as in
 * other functions.
 *
 * Revision 1.2  1999/06/01 02:15:31  francis
 * Making sure that, if two illegal prefixes are used, they don't match.
 *
 * Revision 1.1  1999/06/01 01:51:14  francis
 * Adding functions to get/set/clear attributes.  Reorganzing the
 * to-do list.  Fixing a parser bug introduced I removed support for
 * attributes with no values at release 0.2.
 *
 */

static const char* id="$Id: tagmatch.c,v 1.6 1999/07/24 16:55:28 francis Exp $";

#include <doxml/xml.h>

#include <string.h>

static const char*
safestr(const char* s)
{
  return (s ? s : "(null)");
}

int
doxml_tags_match(const doxml_name* name1,
                 const doxml_name* name2,
                 int prefixes_must_match,
                 int second_is_terminator)
{
  if (!(name1 && name2 && (name1->name) && (name2->name)))
    return 0;

  if ((name1->space) || (name2->space))
    {
      if (!((name1->space) && (name2->space)))
        return 0;

      if (strcmp(name1->space,name2->space))
        return 0;

      if (!strcmp(name1->space,"???")) /* Two illegal prefixes don't match */
        return 0;
    }

  if (prefixes_must_match)
    {
      if ((name1->prefix) || (name2->prefix))
        {
          if (!((name1->prefix) && (name2->prefix)))
            return 0;
          if (strcmp(name1->prefix,name2->prefix))
            return 0;
        }
    }

  if (second_is_terminator)
    {
      if ((name2->name[0])!='/')
        return 0;
      if (strcmp(name1->name,(name2->name)+1))
        return 0;
    }
  else
    if (strcmp(name1->name,name2->name))
      return 0;

  return 1;
}

int
doxml_tags_match_dtd(const doxml_name* name_in_doc,
                     const doxml_name_in_dtd* name_in_dtd)
{
  if (!(   name_in_doc
        && name_in_dtd
        && (name_in_doc->name)
        && (name_in_dtd->name)
        )
      )
    return 0;

  if ((name_in_doc->prefix) || (name_in_dtd->prefix))
    {
      if (!((name_in_doc->prefix) && (name_in_dtd->prefix)))
        return 0;
      if (strcmp(name_in_doc->prefix,name_in_dtd->prefix))
        return 0;
    }

  if (strcmp(name_in_doc->name,name_in_dtd->name))
    return 0;

  return 1;
}

int
doxml_tags_match_dtd2(const doxml_name_in_dtd* name1,
                      const doxml_name_in_dtd* name2)
{
  doxml_name n;
  n.prefix=name1->prefix;
  n.space=0;
  n.name=name1->name;
  return doxml_tags_match_dtd(&n,name2);
}
